/**********************************************************************
 *
 * data_denter_dumper.js
 *
 * (c) 2002 Slaven Rezic. 
 * This program is free software; you can redistribute it and/or modify
 * it under the same terms as Perl.
 *
 **********************************************************************/

function DataDenter(inargs) {
    if (typeof inargs == "undefined") inargs = {};
    // members
    this.width = inargs.width || 4;
    this.comma = inargs.comma || " => ";
    this.level = inargs.level || 0;
    this.tabwidth = inargs.tabwidth || 8;
    // nyi: this.sort
    // nyi: this.maxlines = inargs.maxlines || 0;
    // nyi: this.hashmode = inargs.hashmode || 0;

    // methods
    this.indent = DataDenter_indent;
    this.dump1  = DataDenter_indent;
    this._indent_data   = DataDenter_indent_data;
    this._indent_undef  = DataDenter_indent_undef;
    this._indent_hash   = DataDenter_indent_hash;
    this._indent_array  = DataDenter_indent_array;
    this._indent_value  = DataDenter_indent_value;
    this._print_ref     = DataDenter_print_ref;
}

function DataDenterIndent(arg) {
    var o = new DataDenter();
    return o.indent(arg);
}

function DataDenter_indent(arg) {
    var stream = "";
    this.key = "";
    stream += this._indent_data(arg);
    return stream;
}

// private functions and methods

function DataDenter_indent_data(arg) {
    if        (typeof arg == "undefined" || arg == null) {
	return this._indent_undef(arg);
    } else if (typeof arg == "#array") {
	return this._indent_array(arg);
    } else if (typeof arg == "object") {
	var is_a_mapping = false;
	var is_empty = true;
	for (var i in arg) {
	    is_emoty = true;
	    if (isNaN(i)) {
		is_a_mapping = true;
		break;
	    }
	}
	if (!is_empty) {
	    if (is_a_mapping) {
		return this._indent_hash(arg);
	    } else {
		return this._indent_array(arg);
	    }
	} else {
	    if (typeof arg.length != "undefined") {
		return this._indent_array(arg);
	    } else {
		return this._indent_hash(arg);
	    }
	}
    } else if (typeof arg != "string") {
	return this._indent_value(String(arg)); // force string
    } else {
	return this._indent_value(arg);
    }
}

function DataDenter_indent_value(data) {
    var stream;
    if (data.match(/\n/)) {
	var marker = "EOV";
	var marker_i = "";
	while (data.match(new RegExp("^" + marker + marker_i + "$"))) { // XXX /m flag?? --- wrong!
	    marker_i++;
	}
	marker = marker + String(marker_i);
	var chomp = "";
	if (data.substr(data.length-1, 1) != "\n") {
	    chomp = "-";
	}
	stream = "<<" + marker + chomp + "\n";
	stream += this.key; this.key = "";
	var data_array = data.split("\n");
	data = "";
	for(var i = 0; i < data_array.length; i++) {
	    data += escape(data_array[i]) + "\n";
	}
	if (chomp == "") data = data.substr(0, data.length-1);
	stream += data + marker + "\n";
    } else if (data.match(/^[\s\%\@\$\\?\"]|\s$/) ||
	       data.match(new RegExp(this.comma)) || // XXX mask regexp!
	       data.match(/([\x00-\x1f\x7f-\xff])/) ||
	       data == "") {
	data = escape(data);
	stream = "\"" + data + "\"" + "\n";
	stream += this.key; this.key = "";
    } else {
	stream = data + "\n";
	stream += this.key; this.key = "";
    }
    return stream;
}

function DataDenter_indent_hash(data) {
    var stream = this._print_ref("%");
    var indent = ++this.level * this.width;
    for(var key in data) {
	var key_out = key;
	if (key.match(/\n/) ||
	    key.match(new RegExp(this.comma)) // XXX mask regexp!
	   ) {
	    var marker = "EOK";
	    var marker_i = "";
	    while (key.match(new RegExp("^" + marker + String(marker_i) + "$"))) { //XXX /m flag? --- wrong!
		marker_i++;
	    }
	    marker = marker + String(marker_i);
	    var chomp = "";
	    if (key.substr(key.length-1, 1) != "\n") { //XXX wrong in perl version???
		chomp = "-";
		this.key = key;
	    } else {
		this.key = key.substr(0, key.length-1);
	    }
	    this.key += "\n" + marker + "\n";
	    key_out = "<<" + marker + chomp;
	} else if (key.match(/^[\s\%\@\$\\?\"]|\s$/)) {
	    key_out = "\"" + key + "\"";
	}
	stream += DataDenter_x(" ", indent) + key_out + this.comma;
	stream += this._indent_data(data[key]);
    }
    this.level--;
    return stream;
}

function DataDenter_indent_array(data) {
    var stream = this._print_ref("@");
    var indent = ++this.level * this.width;
    for(var i = 0; i < data.length; i++) {
	stream += DataDenter_x(" ", indent);
	stream += this._indent_data(data[i]);
    }
    this.level--;
    return stream;
}

function DataDenter_indent_undef(data) {
    var stream = "?\n";
    stream += this.key; this.key = "";
    return stream;
}

function DataDenter_print_ref(symbol) {
    var stream = symbol + "\n";
    stream += this.key; this.key = "";
    return stream;
}

function DataDenter_x(s,n) {
    var ret = "";
    for (var i=1; i<=n; i++) {
	ret += s;
    }
    return ret;
}
