#!/usr/bin/perl -w
# -*- perl -*-

#
# $Id: 10dump.t,v 1.3 2003/02/08 22:01:43 eserte Exp $
# Author: Slaven Rezic
#

use strict;
use YAML;

BEGIN {
    if (!eval q{
	use Data::Compare;
	use Data::JavaScript 1.07 {UNDEF=>'null'};
	use JavaScript;
	use Test;
	1;
    }) {
	print "1..0 # skip: testing needs Test, Data::Compare, Data::JavaScript1.07 and JavaScript modules\n";
	exit;
    }
}

# A hack so that the tests can have a place to put stuff:
use vars qw(*joe_random_global);
my $current_test = 1;
my $output = '';

#$Data::JavaScript::UNDEF_AS_NULL = 1;

my $runtime = new JavaScript::Runtime();
my $context = $runtime->create_context();

$context->eval_file("yaml_dumper.js");

test_round_trip(<DATA>);

sub test_round_trip {
    my (@tests_as_yaml) = @_;
    for (pick_which_tests_to_run(@tests_as_yaml)) {
        eval {
            run_round_trip_case($_);
        }; if ($@) {
            $output .= "not ok " . $current_test++ . "\n";
            $output .= $@;
        }
    }
    my $total_tests = $current_test - 1;
    print "1..$total_tests\n$output";
}

sub run_round_trip_case {
    $_ = shift;
    my ($javascript_pre, $javascript_post, $javascript_input, $yaml_input, $config) = @$_{qw(javascript-pre javascript-post javascript yaml config)};

    # Make sure this gets reset:
    local *joe_random_global;

    (my $pseudo_id = $javascript_input) =~ s/\s+/ /g;

    $config ||= '';

    if (defined $javascript_pre) {
	$context->eval($javascript_pre);
    }

    if ($javascript_input =~ s/\A\s*\((.*)\)\s*\z/[$1]/s) {
	$javascript_input = "o=".$javascript_input.";o;";
    } else {
	$javascript_input = "o=[".$javascript_input."];o;";
    }
    my @javascript_struct = $context->eval("o=null;\n" . $javascript_input);
    die $@ if $@;

    my $first_dump = $context->eval("yaml=new YAML(); yaml.dump(o)");
    die $@ if $@;

    is($first_dump, $yaml_input, "manual: $pseudo_id");

    if (defined $javascript_post) {
	$context->eval($javascript_post);
    }

    return if $_->{'no-round-trip'};

    my @loaded = YAML::Load($first_dump);
    # TODO: Add another assertion here

    my $second_dump = eval( $config . '; YAML::Dump(@loaded)' );
    die $@ if $@;

    is($second_dump, $first_dump, "round-tripped: $pseudo_id");
}

sub pick_which_tests_to_run {
    my @test_objects = YAML::Load(join '', @_);
    my @flagged_test_objects = grep { $_->{only} } @test_objects;
    @test_objects = @flagged_test_objects if @flagged_test_objects;
    @test_objects;
}

# Because Test.pm is... umm... "less than featureful".
sub is {
    my ($x, $y, $id) = @_;
    if ($x ne $y) {
        s/\n/\n# /g for $x, $y;
        my $msg = <<EOFAILURE;
#     Failed test ('$id')
#          got: '$x'
#     expected: '$y'
EOFAILURE
        $output .= "not ok $current_test\n$msg";
    } else {
        $output .= "ok $current_test\n" unless $ENV{HUSH};
    }
    $current_test++
}

__DATA__
javascript: |
    {name:'Brian Ingerson',
     rank:'JAPH',
     'serial number':8675309,
    }
yaml: |
    --- #YAML:1.0
    name: Brian Ingerson
    rank: JAPH
    serial number: 8675309
---
javascript: |
     {fruits:['apples', 'oranges', 'pears'],
      meats:['beef', 'pork', 'chicken'],
      vegetables:['carrots', 'peas', 'corn'],
     }
yaml: |
    --- #YAML:1.0
    fruits:
      - apples
      - oranges
      - pears
    meats:
      - beef
      - pork
      - chicken
    vegetables:
      - carrots
      - peas
      - corn
---
javascript: |
    [42, '43', '-44', 45]
yaml: |
    --- #YAML:1.0
    - 42
    - 43
    - -44
    - 45
---
javascript: |
    [
     'foo bar',
     'http://www.yaml.org',
     '12:34'
    ]
yaml: |
    --- #YAML:1.0
    - foo bar
    - http://www.yaml.org
    - 12:34
---
javascript: |
    (1, " foo ", "bar\n", [], {})
yaml: |
    --- #YAML:1.0 1
    --- #YAML:1.0 ' foo '
    --- #YAML:1.0 "bar\n"
    --- #YAML:1.0 []
    --- #YAML:1.0 {}
---
javascript: |
    '8\'-0" x 24" Lightweight'
yaml: |
    --- #YAML:1.0 8'-0" x 24" Lightweight
---
javascript: |
    "foo\0bar"
yaml: |
    --- #YAML:1.0 "foo\zbar"
---
javascript-pre: |
    save_YAML_Indent=YAML_Indent;
    YAML_Indent=1;
javascript-post: |
    YAML_Indent=save_YAML_Indent;
javascript: |
    [{foo: 42, bar: 44}]
yaml: |
    --- #YAML:1.0
    - bar: 44
      foo: 42
---
javascript-pre: |
    save_YAML_Indent=YAML_Indent;
    YAML_Indent=4;
javascript-post: |
    YAML_Indent=save_YAML_Indent;
javascript: |
    [{foo: 42, bar: 44}]
yaml: |
    --- #YAML:1.0
    - bar: 44
      foo: 42
---
javascript-pre: |
    rx = new RegExp("bozo$", "i"); // "
javascript: |
    [rx]
yaml: |
    --- #YAML:1.0
    - !javascript/regexp:
      MODIFIERS: i
      REGEXP: bozo$
---
javascript: |
    [null, null]
yaml: |
    --- #YAML:1.0
    - ~
    - ~
---
javascript: |
    ['']
yaml: |
    --- #YAML:1.0
    - ''
#---
#javascript: |
#    [
#        1/100000000,
#        10**20,
#        -10**20,
#    ]
#yaml: |
#    --- #YAML:1.0
#    - 1e-08
#    - 1e+20
#    - -1e+20
---
javascript: |
    [
        23, 
        3.45, 
        123456789012345, 
    ]
yaml: |
    --- #YAML:1.0
    - 23
    - 3.45
    - 123456789012345
#---
#javascript: |
#    $joe_random_global = "monkey";
#    [bless \$joe_random_global, "Banana"]
#yaml: |
#    --- #YAML:1.0 
#    - !javascript/$Bananas monkey
---
javascript: |
    {'foo: bar':'baz # boo','foo ':'  monkey', }
yaml: |
    --- #YAML:1.0
    'foo ': '  monkey'
    'foo: bar': 'baz # boo'
#---
#XXX problem with YAML.pm 0.35
#  javascript: |
#      "EOVtest\nEOV0\nEOV1"
#  yaml: |+
#      --- #YAML:1.0 "EOVtest\nEOV0\nEOV1"
