#!/usr/bin/perl -w
# -*- perl -*-

#
# $Id: data_denter.t,v 1.3 2003/02/08 22:01:43 eserte Exp $
# Author: Slaven Rezic
#

use strict;
use Data::Dumper;

BEGIN {
    if (!eval q{
	use Test;
	use Data::Compare;
	use Data::JavaScript 1.07 {UNDEF=>'null'};
	use JavaScript;
	use Data::Denter 0.16; # yet a private modification
	1;
    }) {
	print "1..0 # skip: no Test, Data::Compare, Data::JavaScript 1.07, JavaScript and Data::Denter 0.16 modules\n";
	exit;
    }
}

BEGIN { plan tests => 4 }

#$Data::JavaScript::UNDEF_AS_NULL = 1;

my $runtime = new JavaScript::Runtime();
my $context = $runtime->create_context();

# for debugging:
$context->bind_function(name => "warn",
			func => sub { warn "@_" });

$context->eval_file("data_denter_dumper.js");

my @data = (
    ["simple"],

    [1,0,undef,
     "   01234567890123456789012345678901234567890123456789012345678901234567890",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n\n\n",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n",
     "0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n",
     1,2,3,"string",{foo=>"bar",x=>"zah"},4,5,6,
     "EOVtest\nEOV0\nEOV1",
     [],
     {},
#     qr/^foo.*bar$/g,
    ],

    {foo => "bar",
     x => "zah\nnewline\n",
     "\"com\nplex:\"" => [1,2,3,{foo => "bar", x => "zah"},4,5,6],
    },

    join("", map { chr($_) } (0..255)),
);

my $nr_test = 0;
foreach my $d (@data) {
    $nr_test++;

    my $jsdump = jsdump('data', $d);
    $context->eval($jsdump);
    my $dddump = $context->eval('dd = new DataDenter(); dddump = dd.indent(data); dddump;');
    my $d2;
    eval {
	$d2 = Data::Denter::Undent($dddump);
1;
    };
    if (!$d2) {
	my $line = 1;
	warn join("\n", map { sprintf("%3d: %s", $line++, $_) } split /\n/, $dddump);
	warn $@;
	ok(0);
	next;
    }
    ok(Compare($d, $d2), 1,
       Data::Dumper->new([$d,$d2],[])->Indent(1)->Useqq(1)->Dump);
}

__END__
