#!/usr/bin/perl -w
# -*- perl -*-

#
# $Id: mozilla-js.t,v 1.6 2003/02/08 22:01:44 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2002 Slaven Rezic. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://www.rezic.de/eserte/
#

use strict;
use YAML;
use Data::Dumper;

BEGIN {
    if (!eval q{
	use Data::Compare;
	use Data::JavaScript 1.07 {UNDEF=>'null'};
	use JavaScript;
	use Test;
	1;
    }) {
	print "1..0 # skip: testing needs Test, Data::Compare, Data::JavaScript 1.07 and JavaScript modules\n";
	exit;
    }
}


BEGIN { plan tests => 4 };

#$Data::JavaScript::UNDEF_AS_NULL = 1;

my $runtime = new JavaScript::Runtime();
my $context = $runtime->create_context();

$context->eval_file("yaml_dumper.js");

my @data = (
    [1,0,undef,
     "   01234567890123456789012345678901234567890123456789012345678901234567890",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n\n\n",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n",
     "0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n",
     1,2,3,"string",{foo=>"bar",x=>"zah"},4,5,6,
     "EOVtest\nEOV0\nEOV1",
     [],
     {},
#     qr/^foo.*bar$/g,
    ],

    {foo => "bar",
     x => "zah\nnewline\n",
     "\"com\nplex:\"" => [1,2,3,{foo => "bar", x => "zah"},4,5,6],
    },

    join("", map { chr($_) } (0..255)),
);

foreach my $d (@data) {
    my $jsdump = jsdump('data', $d);
    $context->eval($jsdump);
    my $jsyaml = $context->eval('yaml = new YAML(); yaml.dump([data]);');
    my $d2;
    eval {
	$d2 = Load($jsyaml);
    };
    if (!$d2) {
	my $line = 1;
	warn join("\n", map { $line++.": $_" } split /\n/, $jsyaml);
	die $@;
    }
    ok(Compare($d, $d2), 1,
       Data::Dumper->new([$d,$d2],[])->Indent(1)->Useqq(1)->Dump);
}

{
    # check dump1
    my $jsdump = jsdump('data', $data[0]);
    my $jsyaml1 = $context->eval("$jsdump; yaml = new YAML(); yaml.dump1(data)");
    my $jsyaml2 = $context->eval("yaml.dump([data])");
    ok($jsyaml1, $jsyaml2);
}

__END__
