#!/usr/bin/perl -w
# -*- perl -*-

#
# $Id: njs.t,v 1.3 2002/11/06 12:20:01 eserte Exp $
# Author: Slaven Rezic
#
# Copyright (C) 2002 Slaven Rezic. All rights reserved.
# This program is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: slaven.rezic@berlin.de
# WWW:  http://www.rezic.de/eserte/
#

use strict;
use YAML;
use Data::Dumper;
use IPC::Open2;
use File::Temp qw/ :POSIX /;

# REPO BEGIN
# REPO NAME file_name_is_absolute /home/e/eserte/src/repository 
# REPO MD5 d1ade85d511edcaf0d20123948126710
BEGIN {
    if (eval { require File::Spec; defined &File::Spec::file_name_is_absolute }) {
	*file_name_is_absolute = \&File::Spec::file_name_is_absolute;
    } else {
	*file_name_is_absolute = sub {
	    my $file = shift;
	    my $r;
	    if ($^O eq 'MSWin32') {
		$r = ($file =~ m;^([a-z]:(/|\\)|\\\\|//);i);
	    } else {
		$r = ($file =~ m|^/|);
	    }
	    $r;
	};
    }
}
# REPO END

# REPO BEGIN
# REPO NAME is_in_path /home/e/eserte/src/repository 
# REPO MD5 1b42243230d92021e6c361e37c9771d1
sub is_in_path {
    my($prog) = @_;
    return $prog if (file_name_is_absolute($prog) and -f $prog and -x $prog);
    require Config;
    my $sep = $Config::Config{'path_sep'} || ':';
    foreach (split(/$sep/o, $ENV{PATH})) {
	if ($^O eq 'MSWin32') {
	    return "$_\\$prog"
		if (-x "$_\\$prog.bat" ||
		    -x "$_\\$prog.com" ||
		    -x "$_\\$prog.exe");
	} else {
	    return "$_/$prog" if (-x "$_/$prog");
	}
    }
    undef;
}
# REPO END

BEGIN {
    if (!eval q{
	use Test;
	1;
    }) {
	print "1..0 # skip: testing needs Test, Data::Compare, Data::JavaScript1.07 and JavaScript modules\n";
	exit;
    }
}

BEGIN {
    if (!is_in_path("js")) {
	print "1..0 # skip: no javascript interpreter js in PATH\n";
	exit;
    }
}

# todo: poor introspection with njs on Regexps, perl and js subroutine
# outputs are incompatible
BEGIN { plan tests => 4, todo => [1] };

my($js_fh, $js_temp) = tmpnam();
print $js_fh <DATA>;
close $js_fh;

my $pid = open2(\*RDRFH, \*WTRFH, "js --load yaml_dumper.js -f $js_temp")
    or die $!;
print <DATA>;
print WTRFH <DATA>;
close WTRFH;
my $js_yaml;
{
    local $/ = undef;
    $js_yaml = <RDRFH>;
}

# perl test code
my $a = [sub { warn }, 1, 0, qr/(?i:^foo.*bar$)/, "EOVtest\nEOV0\nEOV1"];
my $b = [undef,
	 "   01234567890123456789012345678901234567890123456789012345678901234567890",
	 "   01234567890123456789012345678901234567890123456789012345678901234567890\n\n\n",
	 "   01234567890123456789012345678901234567890123456789012345678901234567890\n",
	 "0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n",
	 1,2,3,"string",undef,{foo=>"bar",x=>"zah"},4,5,6,[],{}];
my $c = {foo => "bar",
	 x => "zah\nnewline\n",
	 "\"com\nplex:\"" => [1,2,3,{foo => "bar", x => "zah"},4,5,6]};
my $d = join("", map { chr($_) } (0..255));
my $perl_yaml = YAML::Dump($a, $b, $c, $d);

my @js_yaml;
my @perl_yaml;

while($js_yaml   =~ /^(--- #YAML:1.0.+?)(?=--- #YAML:1.0|\z)/smg) {
      push @js_yaml, $1;
}
while($perl_yaml =~ /^(--- #YAML:1.0.+?)(?=--- #YAML:1.0|\z)/smg) {
      push @perl_yaml, $1;
}

#require Data::Dumper; print STDERR "Line " . __LINE__ . ", File: " . __FILE__ . "\n" . Data::Dumper->new([\@js_yaml,\@perl_yaml],[])->Indent(1)->Useqq(1)->Dump; # XXX

for my $i (0 .. $#js_yaml) {
    if ($js_yaml[$i] ne $perl_yaml[$i]) {
	ok(0);
	show_diff($js_yaml[$i], $perl_yaml[$i]);
    } else {
	ok(1);
    }
}

END {
    unlink $js_temp if defined $js_temp;
}

sub show_diff {
    return if !is_in_path("diff");

    my($s1,$s2) = @_;
    my $tmpdir = tmpdir();
    my $base   = "$tmpdir/test.$$";

    open(S1, ">$base.1") or die $!;
    print S1 $s1;
    close S1;
    open(S2, ">$base.2") or die $!;
    print S2 $s2;
    close S2;

    open(DIFF, "diff -u $base.1 $base.2 |");
    while(<DIFF>) {
	print "# $_";
    }
    close DIFF;

    unlink "$base.1";
    unlink "$base.2";
}

sub tmpdir {
    foreach my $d ($ENV{TMPDIR}, $ENV{TEMP},
		   "/tmp", "/var/tmp", "/usr/tmp", "/temp") {
	next if !defined $d;
	next if !-d $d || !-w $d;
	return $d;
    }
    undef;
}

__END__
// javascript test code
function foo(x,y) {
    x = 1+y;
    y += x;
}

var a = [foo,true,false,new RegExp("^foo.*bar$", "i"),"EOVtest\nEOV0\nEOV1"]; // no equivalent in perl
var b = [undefined,
	 "   01234567890123456789012345678901234567890123456789012345678901234567890",
	 "   01234567890123456789012345678901234567890123456789012345678901234567890\n\n\n",
	 "   01234567890123456789012345678901234567890123456789012345678901234567890\n",
	 "0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n",
	 1,2,3,"string",null,{foo:"bar",x:"zah"},4,5,6,[],{}];
var c = {foo:"bar",x:"zah\nnewline\n","\"com\nplex:\"":[1,2,3,{foo:"bar",x:"zah"},4,5,6]};

var d = "";
for(var ch=0;ch<256;ch++) {
    d += String.fromCharCode(ch);
}

System.print(YAMLDump(a, b, c, d));
