#!/usr/bin/perl -w
# -*- perl -*-

#
# $Id: perl_dumper.t,v 1.3 2003/02/08 22:01:44 eserte Exp $
# Author: Slaven Rezic
#

use strict;
use Data::Dumper;
use Safe;

BEGIN {
    if (!eval q{
	use Test;
	use Data::Compare;
	use Data::JavaScript 1.07 {UNDEF=>'null'};
	use JavaScript;
	1;
    }) {
	print "1..0 # skip: no Test, Data::Compare, Data::JavaScript 1.07, and JavaScript modules\n";
	exit;
    }
}

BEGIN { plan tests => 5 }

#$Data::JavaScript::UNDEF_AS_NULL = 1;

my $runtime = new JavaScript::Runtime();
my $context = $runtime->create_context();

# for debugging:
$context->bind_function(name => "warn",
			func => sub { warn "@_" });

$context->eval_file("perl_dumper.js");

my @data = (
    ["simple"],

    [1,0,
     #undef,
     "   01234567890123456789012345678901234567890123456789012345678901234567890",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n\n\n",
     "   01234567890123456789012345678901234567890123456789012345678901234567890\n",
     "0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n0123456789\n",
     1,2,3,"string",{foo=>"bar",x=>"zah"},4,5,6,
     "EOVtest\nEOV0\nEOV1",
#     [],
#     {},
#     qr/^foo.*bar$/g,
    ],

    {foo => "bar",
     x => "zah\nnewline\n",
     "\"com\nplex:\"" => [1,2,3,{foo => "bar", x => "zah"},4,5,6],
    },

    [join("", map { chr($_) } (0..255))],
);

my $safe = Safe->new;

foreach my $d (@data) {
    my $jsdump = jsdump('data', $d);
    $context->eval($jsdump);
    my $pddump = $context->eval('pd = new PerlDumper(); pddump = pd.dump(data); pddump;');
    my $d2 = $safe->reval($pddump);
    if (!$d2) {
	my $line = 1;
	warn join("\n", map { sprintf("%3d: %s", $line++, $_) } split /\n/, $pddump);
	warn $@;
	ok(0);
	next;
    }
    ok(Compare($d, $d2), 1,
       Data::Dumper->new([$d,$d2],[])->Indent(1)->Useqq(1)->Dump);
}

{
    # pretty print
    my $jsdump = jsdump('data', $data[1]);
    $context->eval($jsdump);
    my $pddump = $context->eval('pd = new PerlDumper(); pd.PrettyPrint = true; pddump = pd.dump(data); pddump;');
    ok($pddump =~ qr/\n.*\n/s); # check for at least two newlines
}

__END__
